//  Test simplex code with explicitly solved tableau.
//  File:  LpTest.java
//
//
//  Data file:
//
//	M	N			 Constraints, nonbasic var's
//	A(0,0) . . . A(0,N)		 Tableau (reals)
//	   :            :
//	A(M,0) . . . A(M,N)
//	u(1) . . . u(M)			 Basic variables (integers)
//	v(1) . . . v(N)			 Non-basic variables (integers)

import java.io.*;
import java.net.*;


public class LpTest
{
  static String dFile;		// Name of data file

  static float A[][];		// The tableau  [M+1][N+1]

  static int    M, N,		// Number of constraints, non-basic variables
            u[], v[];		// Basic, non-basic variables

  static void loadData(String dataf)
   { int tok;	// token status
     try
      { StreamTokenizer data =
	  new StreamTokenizer
	      (new BufferedInputStream(new FileInputStream(dataf)));
	data.parseNumbers();  data.slashSlashComments(true);
	tok = data.nextToken();  M = (int) data.nval;
	tok = data.nextToken();  N = (int) data.nval;
	A = new float[M+1][N+1];
	u = new int[M+1];  v = new int[N+1];
	for ( int i = 0;  i <= M;  i++ )
	 for ( int j = 0;  j <= N;  j++ )
	  { tok = data.nextToken();  A[i][j] = (float) data.nval; }
	for ( int i = 1;  i <= M;  i++ )
	 { tok = data.nextToken();  u[i] = (int) data.nval; }
	for ( int j = 1;  j <= N;  j++ )
	 { tok = data.nextToken();  v[j] = (int) data.nval; }
      }
     catch (IOException ex)
      { System.out.println("Couldn't read data file - "
			  + ex.getMessage());
      }
   }


  //  Run test using data from file specified as command-line argument

  public static void main(String args[])
   { int istat[] = new int[2];   // For status
     dFile = args[0];
     loadData(dFile);				// Set up tableau
     Simplex.simplex(M, N, A, u, v, istat);	// Optimize
     Simplex.results(M, N, A, u, v, istat);	// Report
   }
}