//  Plotting Differential equation solutions.  This is easily adapted
//  to a variety of examples, the present version covers harmonic
//  oscillators.

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public class DEPlot extends Applet
		   implements Runnable, ActionListener
{ Plotter pl;

  Control ctrl;			// Plot controls

  boolean ready = false;	// To synchronize startup

  Object safety;		// For synchronization

  String parm;			// For applet parameters

  Thread PThread;		// For interaction

  int pwd = 512, pht = 512;	// Canvas dimensions

  // Test parameters:

  int   D = 2;		// Space dimensions 
  float K  = 1.0F,	// Spring constant
	L  = 0.0F,	// damping
	t1 = 10.0F;	// integration time

  // Integration control

  int   M = 4,		// Calculate M groups
	S = 8;		// of S steps between printed reports.

  Harmonic harm;	// To set up and carry out solutions.

   public void init()
    { safety = new Object();
      harm = new Harmonic();
      setLayout(new BorderLayout());
      pl = new Plotter();  add("Center", pl);  pl.init(pwd, pht);
      pl.setXlo(0.0F);  pl.setXhi(10.0F);
      pl.setYlo(0.0F);  pl.setYhi(1.0F);
      // Parameters:
      parm = getParameter("K");
      if ( parm != null ) K = Float.valueOf(parm).floatValue();
      parm = getParameter("L");
      if ( parm != null ) L = Float.valueOf(parm).floatValue();
      parm = getParameter("t1");
      if ( parm != null ) t1 = Float.valueOf(parm).floatValue();
            makeControl();
      ready = true;
    }

 //  The control panel ====================================================

   Parameter pD, pK, pL,	// The numeric (gui) parameters
	     pt1, pM, pS;

   Button plotit;		// Triggers plotting

   String ttl = "Damped harmonic oscillator using the modified "+
		"Euler's method. [scaled to fit]";

   void makeControl()
    { ctrl = new Control(2, 3);  add("North", ctrl);
      ctrl.title(ttl);
      pD = new Parameter("D >= 1 (dimensions)", Integer.toString(D));
      ctrl.addItem(pD);					// 1
      pK = new Parameter("K > 0.0 (spring constant)", Float.toString(K));
      ctrl.addItem(pK);					// 2
      pL = new Parameter("L >= 0.0 (damping)", Float.toString(L));
      ctrl.addItem(pL);					// 3
      pM = new Parameter("M >= 1 (groups)", Integer.toString(M));
      ctrl.addItem(pM);					// 4
      pS = new Parameter("S >= 1 (steps/group)", Integer.toString(S));
      ctrl.addItem(pS);					// 5
      plotit = new Button("Plot");
      ctrl.addItem(plotit);				// 6
      plotit.addActionListener(this);
    }


  //  String -> float

  float toFloat(String s)
   { return Float.valueOf(s).floatValue(); }

  //  String -> int

  int  toInt(String s)
   { return Integer.parseInt(s); }


  //  Responding to the "Plot" button

  public void actionPerformed(ActionEvent e)
   {  boolean changed = false;
     int Dx, Mx, Sx;  float Kx, Lx;
     
     Dx = toInt(pD.pValue());
     if ( Dx != D ) { D = Dx;  changed = true; }
     Kx = toFloat(pK.pValue());
     if ( Kx != K ) { K = Kx;  changed = true; }
     Lx = toFloat(pL.pValue());
     if ( Lx != L ) { L = Lx;  changed = true; }
     Mx = toInt(pM.pValue());
     if ( Mx != M ) { M = Mx;  changed = true; }
     Sx = toInt(pS.pValue());
     if ( Sx != S ) { S = Sx;  changed = true; }
     if ( changed ) harm.harmonic(D, K, L, t1, M, S, pl);
     pl.repaint();
   }



   /* Delay for specified time (in msec.) */

   void pause(int delay)
    { if ( delay > 0 )
       {  try { Thread.sleep(delay); }
          catch (InterruptedException ex) { }
       }
    }


   /* Start process	*/

   public void start()
    { while ( !ready  ) pause(200);		/* Wait for init */
      if ( PThread == null )
       { PThread = new Thread(this);
	 PThread.start(); }
    }


   /* Stop process  */

   public void stop()
    { if ( PThread != null )
       synchronized (safety)
	{ PThread.stop();  PThread = null; }
    }


   public void run()
    { pl.repaint();
      System.out.println("Running.");
      harm.harmonic(D, K, L, t1, M, S, pl);
    }

}
 