/*  Class for ringed bodies, e.g., Saturn	*/

import java.awt.*;
import Body;

class RingBody extends Body
 { 


   /* Draw the body with ring system, diameter d0.	*/
   /* Intended for d0 >= 2.				*/

   public void drawBody(Graphics gx, Point p, int d0)
    { int wi, hi, xi, yi,	/* Inner width, height, x-disp, y-disp	*/
          wo, ho, xo, yo;	/* Outer width, height, x-disp, y-disp	*/
      if ( d0 > 0 )			/* Body should be drawn	*/
       {int r0 = d0/2;
        int rx = Math.max((int) Math.round(0.250*d0), 1);
        int rt = Math.max((int) Math.round(0.375*d0), 1);
        xi = r0 + rx;  wi = d0 + 2*rx;
        xo = xi + rt;  wo = d0 + 2*(rx + rt);
        yi = rx;  hi = 2*rx;
        yo = rt;  ho = 2*rt;
        gx.setColor(bclr);				/* Body color	*/
        gx.fillOval(p.x - xo, p.y - yo, wo, ho);	/* Outer ring	*/
        gx.setColor(bg);
        gx.fillOval(p.x - xi, p.y - yi, wi, hi);	/* Inner ring	*/
        gx.setColor(bclr);				/* Body color	*/
        gx.fillOval(p.x-r0, p.y-r0, d0, d0);		/* Disk		*/
       }
    }


   /* Like drawBody, in current color.  Usually used to erase.	*/
   /* This one is useful only for erasing.                      */

   public void eraseBody(Graphics gx, Point p, int d0)
    { int wo, ho, xo, yo;	/* Outer width, height, x-disp, y-disp	*/
      if ( d0 > 0 )		/* Body should be drawn	*/
       { int r0 = d0/2;
         int rx = Math.max((int) Math.round(0.250*d0), 1);
         int rt = Math.max((int) Math.round(0.375*d0), 1);
         xo = r0 + rx + rt;  wo = d0 + 2*(rx + rt);
         yo = rt;  ho = 2*rt;
	 gx.fillOval(p.x - xo, p.y - yo, wo, ho);	/* Outer ring	*/
         gx.fillOval(p.x-r0, p.y-r0, d0, d0);		/* Disk		*/
       }
    }


   /*  Like drawBody above, but erases previously drawn image at p0,	*/
   /*  drawing at same scale as before.					*/
   /*  If `trail' then leaves a permanent, one-point, trail.		*/

   public void moveBody(Graphics gx, Point p, Point p0, int d0, boolean trail)
    { int wi, hi, xi, yi,	/* Inner width, height, x-disp, y-disp	*/
          wo, ho, xo, yo;	/* Outer width, height, x-disp, y-disp	*/
      if ( d0 > 0 )				/* Body was drawn */
       { int r0 = d0/2;
         int rx = Math.max((int) Math.round(0.250*d0), 1);
         int rt = Math.max((int) Math.round(0.375*d0), 1);
         xi = r0 + rx;  wi = d0 + 2*rx;
         xo = xi + rt;  wo = d0 + 2*(rx + rt);
         yi = rx;  hi = 2*rx;
         yo = rt;  ho = 2*rt;
	 gx.setColor(bg);			/* Set background */
	 gx.fillOval(p0.x-xo, p0.y-yo, wo, ho);	/* Outer ring	  */
	 gx.fillOval(p0.x-r0, p0.y-r0, d0, d0);	/* Erase disk	  */
	 gx.setColor(bclr);			/* Body color	  */
         if ( trail )
	    gx.drawRect(p0.x, p0.y, 1, 1);	/* One point	  */
	 gx.fillOval(p.x-xo, p.y-yo, wo, ho);	/* Outer ring	  */
         gx.setColor(bg);
	 gx.fillOval(p.x-xi, p.y-yi, wi, hi);	/* Inner ring	  */
	 gx.setColor(bclr);			/* Body color	  */
	 gx.fillOval(p.x-r0, p.y-r0, d0, d0);	/* Disk		  */
	 p0.x = p.x;  p0.y = p.y;
       }
    }

 }
