/* Interpolation Class */
/* program Interp.java */

import java.io.*;
import java.net.*;
import java.lang.Math;

public class Interp 
{
  public int     N = 0,    /* current coefficient array size */
                 M;    /*    (dtb/dtr)                   */
 
  public  double G[],         /*                     */
	        G0[],         /*  coefficient arrays */
	         H[],         /*                     */
                 dtB,         /* delta-t for bodies  */ 
	         dtR;         /* delta-t for rocket  */
                
  public void init ( int n ) {
    N = n;
    G  = new double [n];
    G0 = new double [n];
    H  = new double [n];
  }  /* end of init function */
	        


  public void set_up ( int m, double dB, double dR) {
  
    M = m;
    dtB = dB;
    dtR = dR;

    if ( N < M ) init (M);  /* check array size */ 
    
    for (int p = 1; p < M; p++ ) 
    {	
	/* 

		    (t - a)*(t - b)^2
   	H[a,b](t) = -----------------
		       (a - b)^2

		    (t - b)^2
	G[a,b](t) = ---------
		    (a - b)^2                */



	G[p]  = ( (p*dtR - dtB)* (p*dtR - dtB) ) / (dtB * dtB);
        H[p]  = ( (p*dtR) * (p*dtR - dtB) * (p*dtR - dtB) ) / ( dtB *dtB);
        G0[p] = G[p] + (  H[p]* 2 / dtB  );
      }  
  }     /* end of set_up function */


  public void interpolate ( double X0[][], double X1[][],
                            double V0[][], double V1[][], 
			    double I[][], int p, int N)
   {
          
          double a, b, c, d;
 
	  a =    G0[p];
          b =     H[p];
	  c =    G0[M-p];
	  d =     H[M-p];

          for (int j = 0; j < N; j += 1)
           {
            for (int i = 0; i < 3; i += 1)
             { 
              I[j][i] = X0[j][i]*a + V0[j][i]*b + X1[j][i]*c - V1[j][i]*d;
	     }  /* end of inner for loop */

           }  /* end of outer for loop */

   }  /* end of interpolate function */     

}  /* end of Interp class */

	






